@@ -14,9 +14,10 @@ License: BSD 3-Clause
 import httpx
 from typing import Any, Dict, List, Optional, Tuple
 
-from api_dock.config import find_remote_config, find_route_mapping, get_database_names, get_remote_names, get_remote_versions, is_route_allowed, is_versioned_remote, load_main_config, resolve_latest_version
+from api_dock.config import find_remote_config, find_route_mapping, get_database_names, get_remote_names, get_remote_versions, get_settings, is_route_allowed, is_versioned_remote, load_main_config, resolve_latest_version
 from api_dock.database_config import find_database_route, get_database_versions, is_versioned_database, load_database_config, resolve_latest_database_version
 from api_dock.sql_builder import build_sql_query, extract_path_parameters
+from api_dock.storage_auth import detect_required_backends, extract_table_metadata_by_backend, extract_table_uris, setup_storage_authentication
 
 
 #
@@ -49,6 +50,7 @@ class RouteMapper:
 
         self.remote_names = get_remote_names(self.config)
         self.database_names = get_database_names(self.config)
+        self.settings = get_settings(self.config)
 
 
     def get_config_metadata(self) -> Dict[str, Any]:
@@ -180,12 +182,22 @@ class RouteMapper:
 
         # Construct full URL
         if final_path:
-            full_url = f"{remote_url.rstrip('/')}/{final_path}"
+            # Optionally add trailing slash to avoid redirects from APIs that require it
+            if self.settings.get("add_trailing_slash", True):
+                path_with_slash = final_path if final_path.endswith('/') else final_path + '/'
+                full_url = f"{remote_url.rstrip('/')}/{path_with_slash}"
+            else:
+                full_url = f"{remote_url.rstrip('/')}/{final_path}"
         else:
             full_url = remote_url.rstrip('/')
 
         # Forward the request
-        async with httpx.AsyncClient() as client:
+        # Configure redirect behavior based on settings
+        # Note: httpx automatically follows redirects but blocks HTTPS->HTTP downgrades for security
+        # The follow_protocol_downgrades setting is documented but may require manual redirect handling
+        follow_redirects = True if self.settings.get("add_trailing_slash", True) else False
+
+        async with httpx.AsyncClient(follow_redirects=follow_redirects) as client:
             try:
                 # Forward request
                 response = await client.request(
@@ -321,12 +333,39 @@ class RouteMapper:
 
             # Execute query and fetch results
             conn = duckdb.connect(database=':memory:')
+
+            # Setup authentication for required storage backends
+            # Detects backends from table URIs (s3://, gs://, azure://, http://, local)
+            # and configures authentication using credential chains
+            table_uris = extract_table_uris(database_config)
+            required_backends = detect_required_backends(table_uris)
+
+            # Extract table metadata (region, auth headers, etc.) grouped by backend
+            backend_metadata = extract_table_metadata_by_backend(database_config)
+
+            # Setup authentication for each backend
+            # This automatically discovers credentials from:
+            # - S3: AWS env vars, config files, IAM roles, SSO (+ region from config or env)
+            # - GCS: GCS env vars, service account files, HMAC keys
+            # - Azure: Azure env vars, managed identity, CLI credentials
+            # - HTTP/HTTPS: httpfs extension (+ custom headers/bearer token from config)
+            #
+            # Metadata from config takes precedence over environment variables
+            # Note: Authentication setup failures are graceful - public files will still work
+            auth_results = setup_storage_authentication(conn, required_backends, backend_metadata)
+
             result = conn.execute(sql_query).fetchall()
             columns = [desc[0] for desc in conn.description] if conn.description else []
             conn.close()
 
-            # Convert to list of dictionaries
-            response_data = [dict(zip(columns, row)) for row in result]
+            # Convert to list of dictionaries with JSON-safe values
+            response_data = []
+            for row in result:
+                row_dict = {}
+                for col, val in zip(columns, row):
+                    # Convert non-JSON-serializable types to strings
+                    row_dict[col] = _make_json_safe(val)
+                response_data.append(row_dict)
 
             return (True, response_data, 200, None)
 
@@ -446,4 +485,34 @@ class RouteMapper:
 
 #
 # INTERNAL
-#
\ No newline at end of file
+#
+def _make_json_safe(value: Any) -> Any:
+    """Convert non-JSON-serializable values to JSON-safe types.
+
+    Handles datetime objects, dates, decimals, and other common types
+    that DuckDB returns but aren't directly JSON serializable.
+
+    Args:
+        value: Value to convert.
+
+    Returns:
+        JSON-safe version of the value.
+    """
+    from datetime import date, datetime
+    from decimal import Decimal
+
+    if value is None:
+        return None
+    elif isinstance(value, (datetime, date)):
+        # Convert datetime/date to ISO format string
+        return value.isoformat()
+    elif isinstance(value, Decimal):
+        # Convert Decimal to float
+        return float(value)
+    elif isinstance(value, bytes):
+        # Convert bytes to base64 string
+        import base64
+        return base64.b64encode(value).decode('utf-8')
+    else:
+        # Return as-is for JSON-safe types (str, int, float, bool, list, dict)
+        return value
\ No newline at end of file
